/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#ifndef BOUNDINGBOX3D_H_
#define BOUNDINGBOX3D_H_

#include <string>

#include <xmltio/xmltio.hpp>

#include "../geometry/Point3D.h"
#include "../geometry/Rotation3D.h"

namespace airobots {

/**
 * Bounding-box in 3D in general position and orientation.
 *
 * The general bounding-box with dimensions width x depth x height is
 * constructed by translating and rotating (via transformation) an
 * axis-aligned bounding-box around its center of mass.
 *
 * @author lziegler
 */
class BoundingBox3D: public Type {

public:

	static std::string getBaseTag();
	static std::string getTemplate();

	virtual bool operator==(const BoundingBox3D &other) const;

	void sanitizeLocation(xmltio::Location&) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, BoundingBox3D &type);

	BoundingBox3D();
	BoundingBox3D(const BoundingBox3D &data);
	BoundingBox3D(std::string generator);
	virtual ~BoundingBox3D();

	/**
	 * The width (along the X axis) of the box.
	 * @param width The width (along the X axis) of the box.
	 * @param unit The unit of the width value.
	 */
	void setWidth(float width, LengthUnit unit);
	/**
	 *The width (along the X axis) of the box.
	 * @param unit The unit of the width value.
	 * @return The width (along the X axis) of the box.
	 */
	float getWidth(LengthUnit unit) const;
	/**
	 *The depth (along the Y axis) of the box.
	 * @param depth The depth (along the Y axis) of the box.
	 * @param unit The unit of the depth value.
	 */
	void setDepth(float depth, LengthUnit unit);
	/**
	 *The depth (along the Y axis) of the box.
	 * @param unit The unit of the depth value.
	 * @return The depth (along the Y axis) of the box.
	 */
	float getDepth(LengthUnit unit) const;
	/**
	 * The height (along the Z axis) of the box.
	 * @param height The height (along the Z axis) of the box.
	 * @param unit The unit of the height value.
	 */
	void setHeight(float height, LengthUnit unit);
	/**
	 * The height (along the Z axis) of the box.
	 * @param unit The unit of the height value.
	 * @return The height (along the Z axis) of the box.
	 */
	float getHeight(LengthUnit unit) const;

	void setOrigin(const Point3D &origin);
	Point3D getOrigin() const;

	void setRotation(const Rotation3D &rotation);
	Rotation3D getRotation() const;

private:
	Point3D size;
	Point3D origin;
	Rotation3D rotation;
};

}

#endif /* BOUNDINGBOX3D_H_ */
