/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#ifndef REGIONDATA_H_
#define REGIONDATA_H_

#include <string>
#include <vector>
#include <list>
#include "../../units/UnitConverter.h"

#include <xmltio/xmltio.hpp>

#include "../../Type.h"

namespace airobots {

/// triple holds three objects of arbitrary type.
template<class _T1, class _T2, class _T3>
struct triple {
	typedef _T1 first_type; ///<  @c first_type is the first bound type
	typedef _T2 second_type; ///<  @c second_type is the second bound type
	typedef _T2 third_type; ///<  @c third_type is the third bound type

	_T1 first; ///< @c first is a copy of the first object
	_T2 second; ///< @c second is a copy of the second object
	_T3 third; ///< @c third is a copy of the third object

	triple() :
		first(), second(), third() {
	}

	/** Two objects may be passed to a @c pair constructor to be copied.  */
	triple(const _T1& __a, const _T2& __b, const _T3& __c) :
		first(__a), second(__b), third(__c) {
	}

	/** There is also a templated copy ctor for the @c pair class itself.  */
	template<class _U1, class _U2, class _U3>
	triple(const triple<_U1, _U2, _U3>& __p) :
		first(__p.first), second(__p.second), third(__p.third) {
	}
};

/// Two pairs of the same type are equal iff their members are equal.
template<class _T1, class _T2, class _T3>
inline bool operator==(const triple<_T1, _T2, _T3>& __x, const triple<_T1, _T2, _T3>& __y) {
	return __x.first == __y.first && __x.second == __y.second && __x.third == __y.third;
}

/// <http://gcc.gnu.org/onlinedocs/libstdc++/20_util/howto.html#pairlt>
template<class _T1, class _T2, class _T3>
inline bool operator<(const triple<_T1, _T2, _T3>& __x, const triple<_T1, _T2, _T3>& __y) {
	return __x.first < __y.first || (!(__y.first < __x.first) && __x.second < __y.second)
			|| (!(__y.first < __x.first) && !(__y.second < __x.second) && __x.third < __y.third);
}

/// Uses @c operator== to find the result.
template<class _T1, class _T2, class _T3>
inline bool operator!=(const triple<_T1, _T2, _T3>& __x, const triple<_T1, _T2, _T3>& __y) {
	return !(__x == __y);
}

/// Uses @c operator< to find the result.
template<class _T1, class _T2, class _T3>
inline bool operator>(const triple<_T1, _T2, _T3>& __x, const triple<_T1, _T2, _T3>& __y) {
	return __y < __x;
}

/// Uses @c operator< to find the result.
template<class _T1, class _T2, class _T3>
inline bool operator<=(const triple<_T1, _T2, _T3>& __x, const triple<_T1, _T2, _T3>& __y) {
	return !(__y < __x);
}

/// Uses @c operator< to find the result.
template<class _T1, class _T2, class _T3>
inline bool operator>=(const triple<_T1, _T2, _T3>& __x, const triple<_T1, _T2, _T3>& __y) {
	return !(__x < __y);
}

/**
 * Defines a plane in 3d.
 *
 * @author lziegler
 */
class Path3DData: public Type {

public:

	enum Scope {
		LOCAL, GLOBAL
	};
	Scope scope;

	std::vector<triple<float, float, float> > polygon;
	std::vector<triple<float, float, float> > scalePolygon(
			std::vector<triple<float, float, float> > poly, double factor) const;

public:

	static std::string getBaseTag();
	static std::string getTemplate();

	std::vector<triple<float, float, float> > getPolygon(LengthUnit unit) const;
	void setPolygon(const std::vector<triple<float, float, float> > &polygon, LengthUnit unit);

	void setScope(const Scope &scope);
	Scope getScope() const;

	virtual bool operator==(const Path3DData &other) const;

	void sanitizeLocation(xmltio::Location&) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, Path3DData &type);

	Path3DData();
	Path3DData(std::string generator);
};

}

#endif /* REGIONDATA_H_ */
