package de.unibi.airobots.btl;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.TypeVariable;

import nu.xom.Document;
import nu.xom.Element;
import nu.xom.ParsingException;

import org.apache.log4j.Logger;

import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.data.common.Timestamp;

/**
 * General base class for every data type in the java btl. Every type is at
 * least specified by a generator and timestamps. Additionally every type class
 * must provide two static methods: {@link #getBaseTag()} that provides the name
 * of the root tag the type is usually used with and
 * {@link #fromElement(Element, Type)} that parses the the contents of the given
 * element into the type instance used as argument. For serialization to xml
 * {@link #fillInto(Element)} is used.
 * 
 * @author jwienke
 */
public abstract class Type {

	private Timestamp timestamp = new Timestamp();
	private String generator = "unknown";
	/**
	 * This is the source xom element, from which the current btl object was
	 * parsed. If the object was not parsed from a xml document but created
	 * manually, this field is null.
	 */
	private Element sourceDocument = null;
	/**
	 * Indicates whether the parsing should try to parse older xml
	 * representations without generating errors.
	 */
	private boolean legacyParsing = false;
	private static Logger logger = Logger.getLogger(Type.class);

	/**
	 * Every type should be default constructible.
	 */
	public Type() {
	}

	/**
	 * Sets the parsing mode for older xml representations that may contain
	 * errors in the generator or timestamp tags.
	 * 
	 * @param legacyParsing
	 *            if <code>true</code>, the parsing will not fail directly if
	 *            there is an error in the mentioned tag and instead try to
	 *            guess good values for them. If <code>false</code> every xml
	 *            error will result in a parsing error.
	 */
	public void setLegacyParsing(boolean legacyParsing) {
		this.legacyParsing = legacyParsing;
	}

	/**
	 * Tells whether a legacy parsing mechanism should be used to fill this
	 * element or not. See {@link #setLegacyParsing(boolean)} for details.
	 * 
	 * 
	 * @return <code>true</code> if legacy parsing shall be enabled, else
	 *         <code>false</code>
	 */
	public boolean isLegacyParsing() {
		return this.legacyParsing;
	}

	/**
	 * Sets the timestamp using a single millisecond value.
	 * 
	 * @param timestamp
	 *            The timestamp as a long value unix time.
	 */
	public final void setTimestamp(long timestamp) {
		this.timestamp = new Timestamp(timestamp);
	}

	/**
	 * Sets the timestamps associated with this type.
	 * 
	 * @param timestamp
	 *            timstamps to set
	 */
	public final void setTimestamp(Timestamp timestamp) {
		if (timestamp == null) {
			throw new IllegalArgumentException("Timestamp must not be null.");
		}
		this.timestamp = timestamp;
	}

	/**
	 * Uses the current system time as new timestamp for this class.
	 */
	public final void setTimestampNow() {
		this.timestamp = new Timestamp();
	}

	/**
	 * Returns the timestamps associated with this type.
	 * 
	 * @return timestamp timstamps
	 */
	public final Timestamp getTimestamp() {
		assert (timestamp != null) : "Parsing must have set the timestamp.";
		return timestamp;
	}

	/**
	 * Sets the generator of this type.
	 * 
	 * @param generator
	 *            the new generator name
	 */
	public final void setGenerator(String generator) {
		if (generator == null) {
			throw new IllegalArgumentException("Generator must not be null.");
		}
		this.generator = generator;
	}

	/**
	 * Returns the generator name of this type.
	 * 
	 * @return the generator name of this type.
	 */
	public final String getGenerator() {
		assert (generator != null) : "Parsing must have set the generator.";
		return generator;
	}

	/**
	 * Returns the tag name of the the root tag this type is usually used with
	 * in xml.
	 * 
	 * @return tag name
	 */
	public static String getBaseTag() {
		return "TYPE";
	}

	/**
	 * Parses the contents of the given element into the type instance.
	 * Therefore element must already be parsed into the base tag of the type.
	 * Subclasses must also provide a method with this name and can use the base
	 * class' implementation.
	 * 
	 * @param element
	 *            element to get the contents of this type from
	 * @param type
	 *            type to fill with the contents
	 */
	public static void fromElement(Element element, Type type) {

		// timestamp
		try {

			Element timestampNode = null;
			try {
				timestampNode = ElementParser.getFirstChildElement(element,
						Timestamp.getBaseTag());
			} catch (IllegalArgumentException e) {
				// ignore this exception if legacy parsing is enabled because it
				// could also be TIMESTAMP
				if (!type.isLegacyParsing()) {
					throw e;
				}
			}

			if (timestampNode == null && type.isLegacyParsing()) {
				timestampNode = ElementParser.getFirstChildElement(element,
						"TIMESTAMPS");
			}

			Timestamp timestamp = new Timestamp();
			Timestamp.fromElement(timestampNode, timestamp,
					type.isLegacyParsing());
			type.setTimestamp(timestamp);
			type.setSourceDocument(element);
		} catch (IllegalArgumentException e) {
			if (type.isLegacyParsing()) {
				logger.warn("No timestamp found in the document, "
						+ "but ignoring this for legacy parsing");
				logger.debug(element.toXML(), e);
			} else {
				throw e;
			}
		}

		// generator
		try {
			Element generatorNode = ElementParser.getFirstChildElement(element,
					"GENERATOR");
			type.setGenerator(ElementParser.getTagValue(generatorNode));
		} catch (IllegalArgumentException e) {
			if (type.isLegacyParsing()) {
				logger.warn("No GENERATOR element found in the document["
						+ element.getLocalName() + "], "
						+ "but ignoring this for legacy parsing.");
				logger.debug(element.toXML(), e);
			} else {
				throw e;
			}
		}

	}

	/**
	 * Parses the contents of the given document into a new instance of type.
	 * 
	 * @param <T>
	 *            The type of the new instance.
	 * @param doc
	 *            document to get the contents of this type from
	 * @param type
	 *            Class object of the new type.
	 * @return The new instance of type.
	 * @throws ParsingException
	 *             When parsing failed.
	 */
	public static <T extends Type> T fromDocument(Document doc, Class<T> type)
			throws ParsingException {

		try {
			T t = type.newInstance();
			Method fromElement = type.getMethod("fromElement", new Class[] {
					Element.class, type });
			fromElement.invoke(null, doc.getRootElement(), t);
			t.setSourceDocument(doc.getRootElement());
			return t;
		} catch (InstantiationException e) {
			logger.fatal("Reflection error. Types must have a default "
					+ "constructor.");
			logger.debug(e);
			assert false : "Types must have a default constructor.";
			return null;
		} catch (IllegalAccessException e) {
			logger.fatal("Reflection error. Types must have a default "
					+ "constructor.");
			logger.debug(e);
			assert false : "Types must have a default constructor.";
			return null;
		} catch (SecurityException e) {
			logger.fatal("Reflection error. Types must have a fromElement "
					+ "method.");
			logger.debug(e);
			assert false : "Types must have a fromElement method.";
			return null;
		} catch (NoSuchMethodException e) {
			logger.fatal("Reflection error. Types must have a fromElement "
					+ "method.");
			logger.debug(e);
			assert false : "Types must have a fromElement method.";
			return null;
		} catch (IllegalArgumentException e) {
			logger.fatal("Reflection error. Types must have a fromElement "
					+ "method.");
			logger.debug(e);
			assert false : "Types must have a fromElement method.";
			return null;
		} catch (InvocationTargetException e) {
			logger.fatal("Reflection error. Method fromElement threw an "
					+ "exception.");
			logger.debug(e.getCause());
			assert false : "Types must have a fromElement method.";
			return null;
		}
	}

	/**
	 * Utility method that creates a xom {@link Document} from this instance.
	 * 
	 * @return document containing xml representation of this instance
	 * @param type
	 *            class object from the desired type.
	 */
	public final Document toDocument(Class<? extends Type> type) {

		try {
			Method getBaseTag = type.getMethod("getBaseTag", new Class[] {});
			String baseTag = (String) getBaseTag.invoke(null);

			Document doc = new Document(new Element(baseTag));
			fillInto(doc.getRootElement());
			return doc;

		} catch (IllegalAccessException e) {
			logger.fatal("Reflection error. Types must have a getBaseTag "
					+ "method.");
			logger.debug(e);
			assert false : "Types must have a getBaseTag method.";
			return null;
		} catch (SecurityException e) {
			logger.fatal("Reflection error. Types must have a getBaseTag "
					+ "method.");
			logger.debug(e);
			assert false : "Types must have a getBaseTag method.";
			return null;
		} catch (NoSuchMethodException e) {
			logger.fatal("Reflection error. Types must have a getBaseTag "
					+ "method.");
			logger.debug(e);
			assert false : "Types must have a getBaseTag method.";
			return null;
		} catch (IllegalArgumentException e) {
			logger.fatal("Reflection error. Types must have a getBaseTag "
					+ "method.");
			logger.debug(e);
			assert false : "Types must have a getBaseTag method.";
			return null;
		} catch (InvocationTargetException e) {
			logger.fatal("Reflection error. Types must have a getBaseTag "
					+ "method.");
			logger.debug(e);
			assert false : "Types must have a getBaseTag method.";
			return null;
		}

	}

	/**
	 * Sanitizes the given xml parent element, so that
	 * {@link Type#fillInto(Element)} is prevented from generating duplicate
	 * {@link Element}s and {@link Attribute}s. This method will be called
	 * automatically when invoking {@link Type#fillInto(Element)}.
	 * 
	 * @param parent
	 *            parent node that will be sanitized.
	 */
	public void sanitizeElement(Element parent) {
		Element child = parent.getFirstChildElement(Timestamp.getBaseTag());
		if (child != null) {
			parent.removeChild(child);
		}
		child = parent.getFirstChildElement("GENERATOR");
		if (child != null) {
			parent.removeChild(child);
		}
	}

	/**
	 * Fills the given parent node with the contents of this type. Subclasses
	 * really should override this method but can use the base class'
	 * implementation to fill in the values of the base class without having to
	 * duplicate the serialization code.
	 * 
	 * @param parent
	 *            parent node that will be modified to contain the information
	 *            stored in this type
	 */
	public void fillInto(Element parent) {

		sanitizeElement(parent);

		// timestamp
		Element timestampElement = new Element(Timestamp.getBaseTag());
		getTimestamp().fillInto(timestampElement);
		parent.appendChild(timestampElement);

		// generator
		Element generatorElement = new Element("GENERATOR");
		generatorElement.appendChild(getGenerator());
		parent.appendChild(generatorElement);

	}

	/**
	 * Getter for the source xom element, from which the current btl object was
	 * parsed. If the object was not parsed from a xml document but created
	 * manually, this method returns <code>null</code>.
	 * 
	 * @return The xom element from which the current btl object was parsed.
	 * @throws NoSourceDocumentException
	 *             Is thrown if this object was created manually and not parsed
	 *             from a xml document.
	 */
	public Element getSourceDocument() throws NoSourceDocumentException {
		if (sourceDocument == null) {
			throw new NoSourceDocumentException();
		}
		return sourceDocument;
	}

	/**
	 * Setter for the source xom element, from which the current btl object was
	 * parsed.
	 * 
	 * @param sourceDocument
	 *            The xom element from which this object was parsed.
	 */
	public void setSourceDocument(Element sourceDocument) {
		this.sourceDocument = sourceDocument;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object obj) {

		if (!(obj instanceof Type)) {
			return false;
		}

		Type other = (Type) obj;

		return (timestamp.equals(other.timestamp))
				&& (generator.equals(other.generator));

	}

	/**
	 * Exception used to declare that the current BTL object was not parsed from
	 * a xml document.
	 * 
	 * @author lziegler
	 */
	public static class NoSourceDocumentException extends Exception {

		private static final long serialVersionUID = 3558849451905177392L;

		/**
		 * Constructor.
		 */
		public NoSourceDocumentException() {
			super("object has no source document. Possible reasons:"
					+ "\n * object was not parsed from a xml document"
					+ "\n * concrete btl type was not implemented properly");
		}
	}
}
