package de.unibi.airobots.btl.data.hardwareinfo;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import nu.xom.Attribute;
import nu.xom.Element;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.units.UnitConverter;
import de.unibi.airobots.btl.units.AngleUnit;

/**
 * Representation of camera position.
 * 
 * @author lziegler
 */
public class CameraPositionData extends Type {

	private double pan = 0;
	private double tilt = 0;
	private double zoom = 0;

	/**
	 * Constructor.
	 */
	public CameraPositionData() {
		super();
	}

	/**
	 * Constructor.
	 */
	public CameraPositionData(double pan, double tilt, double zoom) {
		super();
		this.pan = pan;
		this.tilt = tilt;
		this.zoom = zoom;
	}

	/**
	 * Returns the horizontal angle value as radiant for the new position.
	 * 
	 * @return pan in desired angleUnit
	 */
	public double getPan(AngleUnit aU) {
		return UnitConverter.convert(pan, AngleUnit.RADIAN, aU);
	}

	/**
	 * Sets the horizontal angle value as radiant for the new position.
	 * 
	 * @param pan
	 *            Horizontal angle as radiant (0=forward, angles CCW)
	 */
	public void setPan(double pan, AngleUnit aU) {
		this.pan = UnitConverter.convert(pan, aU ,AngleUnit.RADIAN);
	}

	/**
	 * Returns the vertical angle value as radiant for the new position.
	 * 
	 * @return tilt in desired angleUnit
	 */
	public double getTilt(AngleUnit aU) {
		return UnitConverter.convert(tilt, AngleUnit.RADIAN, aU);
	}

	/**
	 * Sets the vertical angle value as radiant for the new position.
	 * 
	 * @param tilt
	 *            Vertical angle as radiant (0=forward, negative = down)
	 */
	public void setTilt(double tilt, AngleUnit aU) {
		this.tilt = UnitConverter.convert(tilt, aU ,AngleUnit.RADIAN);
	}

	/**
	 * Returns the zoom level of the camera as percent of the total zoom.
	 * 
	 * @return zoom [0..1]
	 */
	public double getZoom() {
		assert (zoom >= 0 && zoom <= 1) : "Zoom must be a percent value [0..1].";
		return zoom;
	}

	/**
	 * Sets the zoom as percent of the total zoom of the camera.
	 * 
	 * @param zoom
	 *            desired zoom [0..1]
	 * @throws IllegalArgumentException
	 *             illegal zoom value
	 */
	public void setZoom(double zoom) throws IllegalArgumentException {
		if (zoom < 0 || zoom > 1) {
			throw new IllegalArgumentException(
					"Zoom must be a value in [0..1] but got " + zoom + ".");
		}
		this.zoom = zoom;
	}

	public static String getBaseTag() {
		return "CAMERAPOSITION";
	}

	public static void fromElement(Element element, CameraPositionData type) {

		Type.fromElement(element, type);

		Element panElement = element.getFirstChildElement("PAN");
		type.setPan(ElementParser.getDoubleAttributeValue(panElement, "rad"),AngleUnit.RADIAN);

		Element tiltElement = element.getFirstChildElement("TILT");
		type.setTilt(ElementParser.getDoubleAttributeValue(tiltElement, "rad"),AngleUnit.RADIAN);

		Element zoomElement = element.getFirstChildElement("ZOOM");

		type.setZoom(ElementParser.getDoubleAttributeValue(zoomElement, "qual"));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		Element cylindricElement = new Element("PAN");
		parent.appendChild(cylindricElement);
		cylindricElement.addAttribute(new Attribute("rad", String
				.valueOf(getPan(AngleUnit.RADIAN))));

		Element azimuthElement = new Element("TILT");
		parent.appendChild(azimuthElement);
		azimuthElement.addAttribute(new Attribute("rad", String
				.valueOf(getTilt(AngleUnit.RADIAN))));

		Element zoomElement = new Element("ZOOM");
		parent.appendChild(zoomElement);
		zoomElement.addAttribute(new Attribute("qual", String
				.valueOf(getZoom())));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "CameraPositionData [pan = " + pan + ", tilt = " + tilt
				+ ", zoom = " + zoom + "]";
	}

}
