package de.unibi.airobots.btl.data.hardwareinfo;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.HashMap;
import java.util.Map;
import de.unibi.airobots.btl.units.*;

public class LaserInfo {

	/**
	 * An enum containing the known laser scanner models.
	 */
	public enum ScannerModel {
		SICK_LMS200, HOKUYO_UBG_04LX_F01
	}

	/**
	 * A {@link HashMap} containing the ranges of each camera model. The values
	 * are ranges as degree values.
	 */
	private static final Map<ScannerModel, LaserInfo> info = new HashMap<ScannerModel, LaserInfo>() {
		private static final long serialVersionUID = -1139374584227504395L;
		{
			put(ScannerModel.SICK_LMS200, new LaserInfo(180, // angle range
					361, // number of scans
					4)); // max distance
			put(ScannerModel.HOKUYO_UBG_04LX_F01, new LaserInfo(270, // angle
																	 // range
					541, // number of scans
					4)); // max distance
		}
	};

	public static class OutOfRangeException extends Exception {
		private static final long serialVersionUID = 6678346277276506877L;

		public OutOfRangeException(String message) {
			super(message);
		}
	}

	/**
	 * Getter for the information about the given laser scanner model.
	 * 
	 * @param model
	 *            The laser scanner model to receive information for.
	 * @return {@link LaserInfo} instance, that describes the scanner.
	 */
	public static LaserInfo getInfo(ScannerModel model) {
		return info.get(model);
	}

	/**
	 * Converts a angle value from degree to radiant representation.
	 * 
	 * @param angle
	 *            The angle as degree value.
	 * @return The angle as radiant value.
	 */
//	public static double degreeToRadiant(double angle) {
//		return angle * Math.PI / 180.0;
//	}

	/**
	 * Converts a angle value from radiant to degree representation.
	 * 
	 * @param angle
	 *            The angle as radiant value.
	 * @return The angle as degree value.
	 */
//	public static double radiantToDegree(double angle) {
//		return angle * 180.0 / Math.PI;
//	}

	/**
	 * Calculates the index of the scan value corresponding to the given angle.
	 * 
	 * @param scannerModel
	 *            Identity of the used laser scanner.
	 * @param angle
	 *            The angle of the requested value. [radiant]
	 * @return The index of the laser scan corresponding to the given angle.
	 * @throws OutOfRangeException
	 *             Is thrown if the given angle is not in the range of the laser
	 *             scanner.
	 */
	public static int calcScanIndex(ScannerModel scannerModel, double angle)
			throws OutOfRangeException {

		// range of the laser scanner
		double rangeRad = info.get(scannerModel).getRange(AngleUnit.RADIAN);
		double rangeHalf = rangeRad / 2.0;

		// Convert target angle to scanner's scale
		double target = angle + rangeHalf;
		if (target < 0 || target > rangeRad) {
			throw new OutOfRangeException(
					"requested angle is out of laser range!");
		}

		// width of each scan
		double binWidth = rangeRad
				/ (double) (info.get(scannerModel).getNumScans() - 1);

		// calculate the requested scan and round it
		long targetScan = Math.round(target / binWidth);

		return (int) targetScan;
	}

	private double range;
	private double maxDistance;
	private int numScans;
	private AngleUnit iAU = AngleUnit.DEGREE;

	public LaserInfo(double range, int numScans, double maxDistance) {
		this.range = range;
		this.maxDistance = maxDistance;
		this.numScans = numScans;
	}

	public double getRange(AngleUnit aU) {
		return UnitConverter.convert(range, iAU, aU);
	}

	public double getMaxDistance() {
		return maxDistance;
	}

	public int getNumScans() {
		return numScans;
	}
}
