package de.unibi.airobots.btl.data.object;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.HashSet;
import java.util.Set;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.data.geometry.PrecisePolygon;
import de.unibi.airobots.btl.data.navigation.PositionData;
import de.unibi.airobots.btl.units.LengthUnit;

/**
 * Results of the object recognition. This class is meant so define the position
 * of the object in the world! The given polygon describes the objects's
 * position in world coordinates! If you want to define the location in image
 * coordinates use {@link ObjectLocationData}.
 * 
 * @author lziegler
 */
public class ObjectPositionData extends ObjectData {

	private static final double DEFAULT_EDGE_LENGTH = 0.1;
	private static final double DEFAULT_RELIABILITY = 1.0;

	public static class Hypothesis extends ObjectData.Hypothesis {

		/**
		 * Getter for the xml base tag used for this (de-)serialization.
		 * 
		 * @return xml base tag
		 */
		public static String getBaseTag() {
			return ObjectData.Hypothesis.getBaseTag();
		}

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void fillInto(Element parent) {
			super.fillInto(parent);
		}

		/**
		 * Fills an {@link ObjectPositionData.Hypothesis} object from a given
		 * XOM {@link Element}.
		 * 
		 * @param hypothesisElement
		 *            The XOM {@link Element} to fill an object from.
		 * @param type
		 *            The {@link ObjectPositionData.Hypothesis} object to fill
		 *            with all the information given by the {@link Element}
		 *            object.
		 * @throws ParsingException
		 */
		public static void fromElement(Element hypothesisElement,
				Hypothesis type) throws ParsingException {

			ObjectData.Hypothesis.fromElement(hypothesisElement, type);

		}

		/**
		 * {@inheritDoc}
		 */
		@Override
		public String toString() {
			return super.toString();

		}
	}

	/**
	 * Position of the object in the sensor's image.
	 */
	private PrecisePolygon polygon;

	private String reference = "";

	private String coordinateKind = "";

	public PrecisePolygon getPolygon() {
		return polygon;
	}

	public void setPolygon(PrecisePolygon polygon) {
		this.polygon = polygon;
	}

	public String getReference() {
		return reference;
	}

	public void setReference(String reference) {
		this.reference = reference;
	}

	public String getCoordinateKind() {
		return coordinateKind;
	}

	public void setCoordinateKind(String coordinateKind) {
		this.coordinateKind = coordinateKind;
	}

	/**
	 * Default constructor.
	 */
	public ObjectPositionData() {
		setLegacyParsing(true);
	}

	public Set<Hypothesis> getLocationHypotheses() {

		HashSet<Hypothesis> set = new HashSet<Hypothesis>();
		for (ObjectData.Hypothesis h : hypotheses) {
			if (h instanceof Hypothesis) {
				set.add((Hypothesis) h);
			}
		}
		return set;
	}

	/**
	 * @deprecated This method is not valid for subclasses of {@link ObjectData}
	 *             . Use {@link #getLocationHypotheses()} instead.
	 */
	@Deprecated
	public Set<ObjectData.Hypothesis> getHypotheses() {
		return super.getHypotheses();
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "OBJECTPOSITION";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		// polygon
		Element polygonElem = new Element(PrecisePolygon.getBaseTag());
		polygon.fillInto(polygonElem);
		polygonElem.addAttribute(new Attribute("kind", coordinateKind));
		polygonElem.addAttribute(new Attribute("ref", reference));

		parent.appendChild(polygonElem);
	}

	/**
	 * Fills an {@link ObjectPositionData} object from a given XOM
	 * {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link ObjectPositionData} object to fill with all the
	 *            information given by the {@link Element} object.
	 * @throws ParsingException
	 */
	public static void fromElement(Element objectElement,
			ObjectPositionData type) throws ParsingException {

		try {

			ObjectData.fromElement(objectElement, type);

			Element polygonElem = objectElement
					.getFirstChildElement(PrecisePolygon.getBaseTag());

			type.setPolygon(PrecisePolygon.fromElement(polygonElem));
			type.setCoordinateKind(polygonElem.getAttributeValue("kind"));
			type.setReference(polygonElem.getAttributeValue("ref"));

			// clean up
			type.clearHypotheses();

			// check hypotheses
			Nodes nodes = objectElement.query(Hypothesis.getBaseTag());
			for (int i = 0; i < nodes.size(); i++) {
				Node node = nodes.get(i);
				if (node instanceof Element) {

					Hypothesis hyp = new Hypothesis();
					Hypothesis.fromElement((Element) node, hyp);
					type.addHypothesis(hyp);
				}
			}

		} catch (NullPointerException ex) {
			// this happens when an element or attribute that is required is
			// not present
			throw new IllegalArgumentException("Missing element or attribute "
					+ "in document.", ex);
		}
	}

	public static ObjectPositionData fromPosition(PositionData position,
			String label) {
		return fromPosition(position, label, DEFAULT_EDGE_LENGTH,
				LengthUnit.METER, DEFAULT_RELIABILITY);
	}

	public static ObjectPositionData fromPosition(PositionData position,
			String label, double reliability) {
		return fromPosition(position, label, DEFAULT_EDGE_LENGTH,
				LengthUnit.METER, reliability);
	}

	public static ObjectPositionData fromPosition(PositionData position,
			String label, double edgeLength, LengthUnit unit) {
		return fromPosition(position, label, edgeLength, unit,
				DEFAULT_RELIABILITY);
	}

	public static ObjectPositionData fromPosition(PositionData position,
			String label, double edgeLength, LengthUnit unit, double reliability) {
		double half = edgeLength / 2.0;
		PrecisePolygon poly = new PrecisePolygon();
		poly.addPoint(position.getX(LengthUnit.METER) - half,
				position.getY(LengthUnit.METER) - half, unit);
		poly.addPoint(position.getX(LengthUnit.METER) - half,
				position.getY(LengthUnit.METER) + half, unit);
		poly.addPoint(position.getX(LengthUnit.METER) + half,
				position.getY(LengthUnit.METER) + half, unit);
		poly.addPoint(position.getX(LengthUnit.METER) + half,
				position.getY(LengthUnit.METER) - half, unit);
		Hypothesis hyp = new Hypothesis();
		hyp.setReliability(reliability);
		hyp.setClassLabel(label);
		ObjectPositionData obj = new ObjectPositionData();
		obj.addHypothesis(hyp);
		obj.setPolygon(poly);

		return obj;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		String out = super.toString().replace("[OBJECT", "[OBJECTPOSITION");
		out = out.replace("]", " Polygon: " + polygon + "]");
		return out;
	}
}
