package de.unibi.airobots.btl.data.person;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.Node;
import nu.xom.Nodes;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.data.geometry.Point3D;
import de.unibi.airobots.btl.units.*;

/**
 * The Body class stores information about Body hypothesis coming from the
 * BodyTracker.
 * 
 * @author: Dennis Leroy Wigand <dwigand@techfak.uni-bielefeld.de>
 */
public class Body extends Type {

	private Point3D centerOfMass, skel_Torso;
	private boolean isSkeletonActive;
	private int kinect_id;
	private AngleUnit iAU = AngleUnit.RADIAN;
	private LengthUnit iLU = LengthUnit.MILLIMETER;

	public Body() {

		isSkeletonActive = false;
		centerOfMass = new Point3D(0, 0, 0, iLU);
		skel_Torso = new Point3D(0, 0, 0, iLU);
		kinect_id = -1;

	}

	public double getAngle(AngleUnit angleUnit) {
		double lengthCOM = getDistance(LengthUnit.MILLIMETER);

		return UnitConverter.convert(
				Math.asin(getCoM().getX(LengthUnit.MILLIMETER) / lengthCOM),
				AngleUnit.RADIAN, angleUnit);
	}

	public double getDistance(LengthUnit lengthUnit) {
		return UnitConverter.convert(
				Math.sqrt(Math.pow(getCoM().getX(LengthUnit.MILLIMETER), 2)
						+ Math.pow(getCoM().getZ(LengthUnit.MILLIMETER), 2)),
				LengthUnit.MILLIMETER, lengthUnit);
	}

	public void setDistanceAndAngle(double distance, LengthUnit lengthUnit,
			double angle, AngleUnit angleUnit) {

		double convAngle = UnitConverter.convert(angle, angleUnit,
				AngleUnit.RADIAN);
		double convDist = UnitConverter.convert(distance, lengthUnit,
				LengthUnit.MILLIMETER);

		centerOfMass
				.setX(Math.sin(convAngle) * convDist, LengthUnit.MILLIMETER);
		centerOfMass
				.setZ(Math.cos(convAngle) * convDist, LengthUnit.MILLIMETER);
	}

	public void setKinectId(int id) {
		kinect_id = id;
	}

	public int getKinectId() {
		return kinect_id;
	}

	public Point3D getCoM() {
		return centerOfMass;
	}

	public void setCoM(Point3D com) {
		centerOfMass = com;
	}

	public Point3D getTorso() {
		return skel_Torso;
	}

	public void setTorso(Point3D joint) {
		skel_Torso = joint;
	}

	public boolean isSkeleton() {
		return isSkeletonActive;
	}

	public void setSkeleton(boolean isSkeleton) {
		isSkeletonActive = isSkeleton;
	}

	public static String getBaseTag() {
		return "BODY";
	}

	/** @{inheritDoc */
	public static void fromElement(Element element, Body body) {

		Type.fromElement(element, body);

		Nodes nodes = element.query("*");
		int nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {

				Element elem = (Element) node;
				if (elem.getLocalName().equals("COM")) {

					double comX = ElementParser.getDoubleAttributeValue(elem,
							"x");
					double comY = ElementParser.getDoubleAttributeValue(elem,
							"y");
					double comZ = ElementParser.getDoubleAttributeValue(elem,
							"z");
					body.setCoM(new Point3D(comX, comY, comZ,
							LengthUnit.MILLIMETER));

				}

				if (elem.getLocalName().equals("SKELETON")) {

					boolean skeletonActive = ElementParser
							.getBooleanAttributeValue(elem, "active");
					body.setSkeleton(skeletonActive);

					int id = ElementParser.getIntAttributeValue(elem,
							"kinectid");
					body.setKinectId(id);

					if (body.isSkeleton()) {
						Elements elems = elem.getChildElements();
						for (int j = 0; j < elems.size(); j++) {
							Element e = elems.get(j);
							if (e.getLocalName().equals("TORSO")) {

								double torsoX = ElementParser
										.getDoubleAttributeValue(e, "x");
								double torsoY = ElementParser
										.getDoubleAttributeValue(e, "y");
								double torsoZ = ElementParser
										.getDoubleAttributeValue(e, "z");
								body.setTorso(new Point3D(torsoX, torsoY,
										torsoZ, LengthUnit.MILLIMETER));

							}
						}
					}

				}

			}

		}

	}

	@Override
	public String toString() {
		if (isSkeleton()) {
			return "Body [com = (" + getCoM().getX(iLU) + ", "
					+ getCoM().getY(iLU) + ", " + getCoM().getZ(iLU)
					+ "), isSkeleton = " + isSkeleton() + ", kinectId = "
					+ getKinectId() + " Torso = (" + getTorso().getX(iLU)
					+ ", " + getTorso().getY(iLU) + ", " + getTorso().getZ(iLU)
					+ ")]";
		}
		return "Body [com = (" + getCoM().getX(iLU) + ", " + getCoM().getY(iLU)
				+ ", " + getCoM().getZ(iLU) + "), isSkeleton = " + isSkeleton()
				+ ", kinectId = " + getKinectId() + "]";
	}

	/** @{inheritDoc */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element comElement = new Element("COM");
		comElement.addAttribute(new Attribute("scope", "GLOBAL"));
		comElement.addAttribute(new Attribute("x", String.valueOf(getCoM()
				.getX(iLU))));
		comElement.addAttribute(new Attribute("y", String.valueOf(getCoM()
				.getY(iLU))));
		comElement.addAttribute(new Attribute("z", String.valueOf(getCoM()
				.getZ(iLU))));

		parent.appendChild(comElement);

		Element skeletonElement = new Element("SKELETON");
		int active = (isSkeleton()) ? 1 : 0;
		skeletonElement.addAttribute(new Attribute("active", String
				.valueOf(active)));
		skeletonElement.addAttribute(new Attribute("kinectid", String
				.valueOf(getKinectId())));

		parent.appendChild(skeletonElement);

		if (isSkeleton()) {
			Element torsoElement = new Element("TORSO");
			torsoElement.addAttribute(new Attribute("scope", "GLOBAL"));
			torsoElement.addAttribute(new Attribute("x", String
					.valueOf(getTorso().getX(iLU))));
			torsoElement.addAttribute(new Attribute("y", String
					.valueOf(getTorso().getY(iLU))));
			torsoElement.addAttribute(new Attribute("z", String
					.valueOf(getTorso().getZ(iLU))));

			skeletonElement.appendChild(torsoElement);
		} else {
			skeletonElement.removeChildren();
		}

	}

}
