package de.unibi.airobots.btl.data.person;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;

/**
 * List of all face identifications found.
 * 
 * @author jwienke
 */
public class FaceIdentificationList extends Type implements
		Iterable<FaceIdentificationList.FaceIdentification> {

	/**
	 * One single result of the face identification.
	 * 
	 * @author jwienke
	 */
	public static class FaceIdentification {

		private int regionX;
		private int regionY;
		private int regionWidth;
		private int regionHeight;
		private Integer classId = null;

		FaceIdentification() {
		}

		public int getRegionX() {
			return regionX;
		}

		public int getRegionY() {
			return regionY;
		}

		public int getRegionWidth() {
			return regionWidth;
		}

		public int getRegionHeight() {
			return regionHeight;
		}

		public Integer getClassId() {
			return classId;
		}

		void setRegionX(int regionX) {
			this.regionX = regionX;
		}

		void setRegionY(int regionY) {
			this.regionY = regionY;
		}

		void setRegionWidth(int regionWidth) {
			this.regionWidth = regionWidth;
		}

		void setRegionHeight(int regionHeight) {
			this.regionHeight = regionHeight;
		}

		void setClassId(Integer classId) {
			this.classId = classId;
		}

		public boolean isIdentified() {
			return (classId != null) && (classId >= 0);
		}

		/**
		 * {@inheritsDoc}
		 */
		@Override
		public String toString() {
			StringBuilder strb = new StringBuilder();
			strb.append("#FACEIDENTIFICATION# ");
			strb.append("identified: " + isIdentified() + "; ");
			strb.append("classId: " + getClassId() + "; ");
			strb.append("regionX: " + regionX + "; ");
			strb.append("regionY: " + regionY + "; ");
			strb.append("width: " + regionWidth + "; ");
			strb.append("height: " + regionHeight + "; ");

			return strb.toString();
		}

	}

	private Set<FaceIdentification> identifications = new HashSet<FaceIdentification>();

	public FaceIdentificationList() {
		// The provided data don't have a timestamp and generator
		setLegacyParsing(true);
	}

	private void addIdentification(FaceIdentification identification) {
		identifications.add(identification);
	}

	public static String getBaseTag() {
		return "FACES";
	}

	public static void fromElement(Element element, FaceIdentificationList list) {

		Type.fromElement(element, list);

		// parse all faces in the document
		Nodes nodes = element.query("FACE");
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);

			// parse the face
			if (node instanceof Element) {
				Element elem = (Element) node;

				FaceIdentification ident = new FaceIdentification();

				Nodes children = elem.query("*");
				for (int j = 0; j < children.size(); j++) {

					if (!(children.get(j) instanceof Element)) {
						continue;
					}
					Element child = (Element) children.get(j);

					if (child.getLocalName().equals("REGION")) {

						Nodes subchildren = child.query("*");
						for (int k = 0; k < subchildren.size(); k++) {

							if (!(subchildren.get(k) instanceof Element)) {
								continue;
							}

							Element subchild = (Element) subchildren.get(k);
							if (subchild.getLocalName().equals("RECTANGLE")) {
								ident.setRegionX(ElementParser
										.getIntAttributeValue(subchild, "x"));
								ident.setRegionY(ElementParser
										.getIntAttributeValue(subchild, "y"));
								ident.setRegionWidth(ElementParser
										.getIntAttributeValue(subchild, "w"));
								ident.setRegionHeight(ElementParser
										.getIntAttributeValue(subchild, "h"));
							}

						}

					} else if (child.getLocalName().equals("CLASS")) {

						String classId = ElementParser.getAttributeValue(child,
								"class-id");
						try {
							ident.setClassId(Integer.parseInt(classId));
						} catch (NumberFormatException e) {
							ident.setClassId(null);
						}

					}

				}

				list.addIdentification(ident);

			}
		}

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		throw new UnsupportedOperationException();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Iterator<FaceIdentification> iterator() {
		return identifications.iterator();
	}

	/**
	 * Returns the number of identification results in this list.
	 * 
	 * @return size >= 0
	 */
	public int size() {
		return identifications.size();
	}

}
